export class Main extends Indicator {

    constructor() {
        super();

        this.addInput("Length1", InputType.int, 3);
        this.addInput("Length2", InputType.int, 10);
        this.addInput("Length3", InputType.int, 18);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            up : this.addBuffer(),
            dn : this.addBuffer(),
            uph1 : this.addBuffer(),
            dnh1 : this.addBuffer(),
        };

        this.arr = {
            Histo : [],
            MaHisto : [],
            Histoh1 : [],
            MaHistoh1 : [],
        };
    }

    onInit() {
        this.buffers.up
            .setShape(Shape.Arrow)
            .setAnchor(Anchor.Top)
            .setColor(Color.AliceBlue)
            .setArrow(ArrowSymbols.ArrowUpWards)
        ;

        this.buffers.dn
            .setShape(Shape.Arrow)
            .setAnchor(Anchor.Bottom)
            .setColor(Color.AliceBlue)
            .setArrow(ArrowSymbols.ArrowDownWards)
        ;

        this.buffers.uph1
            .setShape(Shape.Arrow)
            .setAnchor(Anchor.Top)
            .setColor(Color.DarkViolet)
            .setArrow(ArrowSymbols.ArrowUpWards)
        ;

        this.buffers.dnh1
            .setShape(Shape.Arrow)
            .setAnchor(Anchor.Bottom)
            .setColor(Color.DarkViolet)
            .setArrow(ArrowSymbols.ArrowDownWards)
        ;
    }

    async onUpdate() {
        const {Length1, Length2, Length3, PriceType} = this.getInputs(),
            {up, dn, uph1, dnh1} = this.buffers,
            periodH1 = TimeFrame.toNumber(TimeFrame.H1),
            {Open, Time, Close} = await Bar.load([Bar.Mode.Open, Bar.Mode.Time, Bar.Mode.Close], PriceType),
            {Open : OpenH1, Time : TimeH1, Close : CloseH1} = await Bar.load([Bar.Mode.Open, Bar.Mode.Time, Bar.Mode.Close], PriceType, TimeFrame.H1),
            currentPeriod = TimeFrame.toNumber(Current.TimeFrame),
            barSize = Open.length,
            {Histo, MaHisto, Histoh1, MaHistoh1} = this.arr;

        let limit,
            countedBars = Bar.counted(),
            tmp = 0, i, j, m, list;

        if (countedBars < 0) countedBars = 0;
        if (countedBars > 0) countedBars--;

        limit = (barSize - countedBars) / (periodH1 / currentPeriod) + 1;

        let list1 = await Indicators.MA(Current.Symbol, TimeFrame.H1, Length1, 0, Averaging.Exponential, AppliedPrice.Close, PriceType),
            list2 = await Indicators.MA(Current.Symbol, TimeFrame.H1, Length2, 0, Averaging.Exponential, AppliedPrice.Close, PriceType);
        for (i = 0; i < limit; i++){
            Histoh1[i] = list1[i] - list2[i];
        }

        list = Indicators.MAOnArray(Histoh1, 0, Length3, 0, Averaging.Exponential);
        for (j = 0; j < limit; j++){
            MaHistoh1[j] = list[j];
        }

        for (m = 0; m < limit; m++) {
            if (MaHistoh1[m + 1] <= 0 && MaHistoh1[m] > 0) {
                tmp = TimeH1[m];
                for (j = 0; j < barSize; j++) {
                    if (Time[j] === tmp) {
                        uph1.set((j - periodH1 / currentPeriod + 1), OpenH1[m] - 20 * Current.Point);
                    }
                }
            }
            if (MaHistoh1[m + 1] >= 0 && MaHistoh1[m] < 0) {
                tmp = TimeH1[m];
                for (j = 0; j < barSize; j++) {
                    if (Time[j] === tmp)
                        dnh1.set((j - periodH1 / currentPeriod + 1), OpenH1[m] + 20 * Current.Point);
                }
            }
        }

        countedBars = Bar.counted();
        if (countedBars < 0) countedBars = 0;
        if (countedBars > 0) countedBars--;
        limit = barSize - countedBars;

        list1 = await Indicators.MA(Current.Symbol, 0, Length1, 0, Averaging.Exponential, AppliedPrice.Close, PriceType);
        list2 = await Indicators.MA(Current.Symbol, 0, Length2, 0, Averaging.Exponential, AppliedPrice.Close, PriceType);

        for (i = 0; i < limit; i++) {
            Histo[i] = list1[i] - list2[i];
        }

        list = Indicators.MAOnArray(Histo, 0, Length3, 0, Averaging.Exponential);
        for (j = 0; j < limit; j++) {
            MaHisto[j] = list[j];
        }

        for (m = 0; m < limit; m++) {
            if (MaHisto[m + 1] <= 0 && MaHisto[m] > 0) {
                for (i = m + 1; i < limit; i++) {
                    if (uph1.get(i) !== dnh1.get(i)) {
                        if (uph1.get(i) < dnh1.get(i)){
                            up.set(m, (Open[m] - (20 * Current.Point)));
                        }
                        else
                            break;
                    }
                }
            }
            if (MaHisto[m + 1] >= 0 && MaHisto[m] < 0) {
                for (i = m + 1; i < limit-1; i++) {
                    if (uph1.get(i) !== dnh1.get(i)) {
                        if (uph1.get(i) > dnh1.get(i)){
                            dn.set(m, (Open[m] + (20 * Current.Point)));
                        }
                        else
                            break;
                    }
                }
            }
        }
    }
}
